
#include <linux/time.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/module.h>
#include <linux/device.h>
#include <linux/pci.h>
#include <linux/cdev.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/sched.h>
#include <linux/atomic.h>
#include <linux/random.h>
#include <linux/uaccess.h>

#include <linux/unistd.h>
#define P40_FMT "P40DMA:%s(): "
#define PCIE40_DMA_CLASS "pcie40_dma"
#include "altera_dma.h"


//------------------------------------------------------------------------------------------------------------------------------
static int pk_remap(struct file *filp, struct vm_area_struct *vma)
{
  struct pcie40_dma_state *bk_ptr = filp->private_data;
  int act_spage=bk_ptr->act_spage;
  //dma memory for each spage is mapped separately. Before calling mmap in the user program,
  //the act_spage must be set by ALTERA_CMD_ACTSPAGE command operation.
  int ilink=(act_spage & 0xFF00)>>8;
  int jspage=(act_spage & 0xFF);
  unsigned long mem_addr;
  mem_addr = virt_to_phys(	bk_ptr->spt[ilink][jspage].spage_virt_addr);
  if (remap_pfn_range(vma, vma->vm_start, (mem_addr)>>PAGE_SHIFT,vma->vm_end - vma->vm_start, vma->vm_page_prot))
    return -EAGAIN;

  return 0;
}
//-----------------------------------------------------------------------------------------------------------------------------
static long altera_dma_ioctl (struct file *filp, unsigned int cmd, unsigned long arg)
{
    struct pcie40_dma_state *bk_ptr = filp->private_data;
  int i,retval=0;
  int mininc;
 
  UAR uar;
  uar.arg=arg;
		 	 	 					
    switch (cmd) {
  case ALTERA_IOCX_CLEAR:
#ifdef PKDEB
    dev_info(  &(bk_ptr->pci_dev->dev)      ,"IOCX CLEAR \n");
#endif
    bk_ptr->spage_rp=0;
    bk_ptr->spage_wp=0;
    atomic_set(&bk_ptr->spages_free,S_PAGES);
    atomic_set(&bk_ptr->spages_ready,0);
    for(i=0;i<bk_ptr->link_number;i++) 
      bk_ptr->ists[i]=ioread32((u32 *)(bk_ptr->common->bar0_regs+DFC_SPAGE_ACK_BASE_ADDR+i*4));
            break;
  case ALTERA_IOCX_RSTART:
#ifdef PKDEB
    dev_info(  &(bk_ptr->pci_dev->dev)      ,"IOCX RSTART \n");
#endif
    if(atomic_read(&bk_ptr->spages_free)==BREAK) { //breaking the loop
      uar.sr.spag_ready=BREAK;
      retval=put_user(uar.arg, (long __user *)arg);
      break;
    }
	
    mininc=dma_test(bk_ptr, bk_ptr->pci_dev);        //mininc - number of filled superpages
    //mininc==0 => timeout inside dma_test, no spages ready
    //spage_rp is updated inside dma_test (spage_set call)				                                                 
    uar.sr.spag_ready=mininc;	
						 
    retval=put_user(uar.arg, (long __user *)arg);
    if(!mininc) break;                               //going out without informing shm write         
    atomic_set(&bk_ptr->spages_ready,mininc);        //set parameter for shm write (number of filled spages)
    wake_up(&bk_ptr->wait_w);                        //wake up shm write; 
            break;
  case ALTERA_IOCX_RWAIT:
#ifdef PKDEB
    dev_info(  &(bk_ptr->pci_dev->dev)      ,"IOCX RWAIT \n");
#endif
    wait_event_interruptible(bk_ptr->wait_r, atomic_read(&bk_ptr->spages_free)); //sleep until a space is available
#ifdef PKDEB
    dev_info(  &(bk_ptr->pci_dev->dev)      ,"IOCX RWAIT awoken\n");
#endif
    break;		
  case ALTERA_IOCX_WWAIT:
#ifdef PKDEB
    dev_info(  &(bk_ptr->pci_dev->dev)      ,"IOCX WWAIT \n");
#endif
    wait_event_interruptible(bk_ptr->wait_w, atomic_read(&bk_ptr->spages_ready));		
    uar.sr.spag_ready=atomic_read(&bk_ptr->spages_ready);
    uar.sr.spag_wp	 =bk_ptr->spage_wp;
    atomic_set(&bk_ptr->spages_ready,0);
#ifdef PKDEB
    dev_info(  &(bk_ptr->pci_dev->dev)      ,"IOCX WWAIT awoken! ispages_ready: %d ispage_wp: %d\n",uar.sr.spag_ready,uar.sr.spag_wp);
#endif
    retval=put_user(uar.arg, (long __user *)arg);
    break;				
  case ALTERA_IOCX_RCONT: 
#ifdef PKDEB
    dev_info(  &(bk_ptr->pci_dev->dev)      ,"IOCX RCONT : %zx \n", arg);		
#endif
    bk_ptr->spage_wp=(bk_ptr->spage_wp+uar.sr.spag_ready)%S_PAGES;
    for(i=0;i<bk_ptr->link_number;i++) 	bk_ptr->ists[i]+=uar.sr.spag_ready; //nowe, 23.12.2019
    atomic_set(&bk_ptr->spages_free, uar.sr.spag_ready);
    wake_up(&bk_ptr->wait_r);
    break;
  case ALTERA_IOCX_WCONT: //special action for timeout
#ifdef PKDEB
    dev_info(  &(bk_ptr->pci_dev->dev)      ,"IOCX WCONT : %zx \n", arg);
#endif
    atomic_set(&bk_ptr->spages_ready, BREAK); //special code for breaking the loop; przejdzie do WWAIT i do spag_ready
    wake_up(&bk_ptr->wait_w);
    break;		
  default:
    return(-EINVAL);
       
   }  
  return(retval);
}
//-----------------------------------------------------------------------------------------------------------------------------
ssize_t altera_dma_read(struct file *file, char __user *buf, size_t count, loff_t *pos) {
  if (altera_dma_rw(file, buf, count, pos, 1) < 0)
    return -1;
  return count;
}
//-----------------------------------------------------------------------------------------------------------------------------
ssize_t altera_dma_write(struct file *file, char __user *buf, size_t count, loff_t *pos) {
  if (altera_dma_rw(file, buf, count, pos, 0) < 0)
    return -1;
  return count;
}
//-----------------------------------------------------------------------------------------------------------------------------
ssize_t altera_dma_rw(struct file *file, char __user *buf, size_t count, loff_t *pos, int read) {

  struct dma_cmd __user *ucmd_p = (struct dma_cmd *)buf;
  struct pcie40_dma_state *bk_ptr = file->private_data;
  return altera_dma_exec_cmd(ucmd_p, bk_ptr);
}
//-----------------------------------------------------------------------------------------------------------------------------
ssize_t altera_dma_exec_cmd(struct dma_cmd *ucmd, struct pcie40_dma_state * bk_ptr) {
  int rc, num_input;
  int value=77;
  unsigned av[2];
  struct dma_cmd kcmd;
  //struct dma_status curr;
  if (copy_from_user (&kcmd, ucmd, sizeof(struct dma_cmd))) {
    rc = -EFAULT;
    return rc;
}

  switch (kcmd.cmd) {   

  case   ALTERA_CMD_ACTSPAGE:  {
    //set spage_number for mapping
    if (copy_from_user (&num_input, kcmd.buf, sizeof(int))) {
      rc = -EFAULT;
      return rc;
    }
    bk_ptr->act_spage=num_input;
    dev_info( &(bk_ptr->pci_dev->dev),"PK act_spage : %x\n",bk_ptr->act_spage);
    break;
  }
  case  MODIFY_LINK_MASK: {
    
    if (copy_from_user (&num_input, kcmd.buf, sizeof(int))) {
      rc = -EFAULT;
      return rc;
    }
    bk_ptr->link_mask = num_input;
    //zrobie w user  iowrite32 (bk_ptr->link_mask, (u32*)(bk_ptr->bar[0]+0x404));
    wmb();
    dev_info( &(bk_ptr->pci_dev->dev)      ,"PK link mask set to: %x\n",bk_ptr->link_mask);
    break;
  }                                                                                                     
  case  GET_REG_BAR0:		{
    if (down_interruptible(&bk_ptr->sem)) return -ERESTARTSYS;
    if (copy_from_user (&num_input, kcmd.buf, sizeof(int))) return(-EFAULT);  
    value = ioread32((u32 *)(bk_ptr->common->bar0_regs+num_input));
    rmb();
    if (copy_to_user (kcmd.buf,&value, sizeof(int))) return(-EFAULT);
    up(&bk_ptr->sem);
    //dev_info(  &(bk_ptr->pci_dev->dev)      ,"PK get_reg offset val:%x %x\n",num_input,value);
    break;
  }
  case  SET_REG_BAR0:		{
    if (down_interruptible(&bk_ptr->sem)) {
      dev_info(  &(bk_ptr->pci_dev->dev) ,"PK dupa!!! down_interruptible\n");
      return -ERESTARTSYS;}
    if (copy_from_user (av, kcmd.buf, 2*sizeof(int))) return(-EFAULT);
    //dev_info(  &(bk_ptr->pci_dev->dev)      ,"SET_REG: adr: %x val:%x\n",av[0],av[1]);
    iowrite32 (av[1], (u32*)(bk_ptr->common->bar0_regs+av[0]));
    wmb();
    up(&bk_ptr->sem);
    break;
  }

  case  GET_REG_BAR2:		{
    if (down_interruptible(&bk_ptr->sem)) return -ERESTARTSYS;
    if (copy_from_user (&num_input, kcmd.buf, sizeof(int))) return(-EFAULT);
    dev_info( &(bk_ptr->pci_dev->dev)      ," register %x\n",num_input);
    value = ioread32((u32 *)(bk_ptr->common->bar2_regs+num_input));
    rmb();
    if (copy_to_user (kcmd.buf,&value, sizeof(int))) return(-EFAULT);
    up(&bk_ptr->sem);

    break;
  }
  case  SET_REG_BAR2:		{
    if (down_interruptible(&bk_ptr->sem)) {
      dev_info(  &(bk_ptr->pci_dev->dev) ,"PK dupa!!! down_interruptible\n");
      return -ERESTARTSYS;}
    if (copy_from_user (av, kcmd.buf, 2*sizeof(int))) return(-EFAULT);
    
    iowrite32 (av[1], (u32*)(bk_ptr->common->bar2_regs+av[0]));
    wmb();
    
    up(&bk_ptr->sem);
    break;
  }
  default:
    printk(KERN_DEBUG "command issued from user space doesn't exist %d", kcmd.cmd);
    break;
    }
    return 0;
}
//-----------------------------------------------------------------------------------------------------------------------------
int altera_dma_open(struct inode *inode, struct file *file) {
    struct pcie40_dma_state *bk_ptr = 0;
    printk(P40_DIAG "PCI DMA OPEN\n", P40_PARM);

    bk_ptr = container_of(inode->i_cdev, struct pcie40_dma_state, cdev);
    file->private_data = bk_ptr;
    bk_ptr->user_pid = current->pid;

    return 0;
}
//-----------------------------------------------------------------------------------------------------------------------------


int altera_dma_release(struct inode *inode, struct file *file) {
    return 0;
}

//-----------------------------------------------------------------------------------------------------------------------------
 
struct file_operations altera_dma_fops = {
    .owner          = THIS_MODULE,
  .read           = altera_dma_read,
  .write          = (void *)altera_dma_write,
    .open           = altera_dma_open,
    .release        = altera_dma_release,
    .unlocked_ioctl = altera_dma_ioctl,
  .mmap			= pk_remap,
	
};
static struct class *pcie40_dma_class = NULL;
//-----------------------------------------------------------------------------------------------------------------------------


static int __init init_chrdev (struct pcie40_dma_state *bk_ptr) {
    int dev_minor = 0;
    int dev_major = 0;
    int devno = 0;
    int rc =-1;
  
   rc = alloc_chrdev_region(&bk_ptr->cdevno, dev_minor, 1, ALTERA_DMA_DEVFILE);
   if (rc < 0) {
      printk(P40_ERR "alloc_chrdev_region()\n", P40_PARM);
      goto err_alloc_chrdev_region;
    }
    dev_major = MAJOR(bk_ptr->cdevno);
    if (dev_major < 0) {
      printk(P40_ERR "cannot get major ID %d",P40_PARM, dev_major);
    }
    
    devno = MKDEV(dev_major, dev_minor);
   
    rc = pcie40_setup_cdev(pcie40_dma_class, &(bk_ptr->cdev), devno, dev_minor, 0, ALTERA_DMA_DEVFILE, bk_ptr->common->dev_id, &altera_dma_fops);
    if (rc < 0) {
      goto err_bar0_dev;
    }
   return 0;
  err_bar0_dev:
    unregister_chrdev_region(bk_ptr->cdevno, 1);
  err_alloc_chrdev_region:
    return rc;
    
}



//-----------------------------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------------------
static int dma_test(struct pcie40_dma_state *bk_ptr, struct pci_dev *dev)
{
  int timeout=10000; //mini timeout   
  unsigned found=~(bk_ptr->link_mask);
  static int i;
  static unsigned int osts[LINK_NUMBER];
  static int		    spageinc[LINK_NUMBER];
  static int mininc,bitsel;
      


#ifdef PKDEB
  dev_info(&dev->dev,"DMA TEST\n");
#endif
  while(down_interruptible(&bk_ptr->sem)) cpu_relax();
  spage_set(bk_ptr); //spages_free is cleared inside, r pointer incremented	    
  for(i=0;i<bk_ptr->link_number;i++) spageinc[i]=10000000; //large value
  while (1) {				
    for(i=0,bitsel=1;i<bk_ptr->link_number;i++,bitsel=(bitsel<<1)) {
      if(found & bitsel) continue;
      osts[i] = ioread32((u32 *)(bk_ptr->common->bar0_regs+DFC_SPAGE_ACK_BASE_ADDR+i*4));
    
	  rmb();
      if((spageinc[i]=osts[i]-bk_ptr->ists[i])) {
	//#ifdef PKDEB
	if (( osts[i] -bk_ptr->ists[i]) > 1) 
          {dev_info(&dev->dev,"lost paquets ? checking: %d ists: %x osts:%x\n",i,bk_ptr->ists[i],osts[i]);
        
	}
	//#endif
	found |= bitsel;
    }
}
    if(found==ALL_OK || !timeout) break;
    timeout--;					
    schedule();
    //cpu_relax();	
  } //while

  if(!timeout) {  mininc=0;
 #ifdef PKDEB
    dev_info(&dev->dev,"Timeout \n");
#endif
}
  else
{
      //szukam najmniejszego przyrostu, nieczynne sa tak duze, ze nie uwzglednimy
      for(i=0,mininc=100;i<bk_ptr->link_number;i++) if(mininc>spageinc[i]) mininc=spageinc[i];
#ifdef PKDEB
      dev_info(&dev->dev,"dma_test: mininc: %d %d\n",mininc,spageinc[0]);
#endif
      //	for(i=0;i<bk_ptr->link_number;i++) 	bk_ptr->ists[i]+=mininc;
	 }
    
  up(&bk_ptr->sem);		  
  return(mininc);
	}
	
	
//-----------------------------------------------------------------------------------------------------------------------------
static int spage_set(struct pcie40_dma_state *bk_ptr) {
  int i,link,spage,ispages_free=atomic_read(&bk_ptr->spages_free);
  unsigned bitsel;	
#ifdef PKDEB
  struct pci_dev *dev=bk_ptr->pci_dev;
  dev_info(&dev->dev, "PK ispages_free: %d\n",ispages_free);
#endif
  if(!ispages_free) return(0);
  for(link=0,bitsel=1;link<bk_ptr->link_number;link++,bitsel=(bitsel<<1)) {
    if(( (bk_ptr->link_mask) & bitsel))
      for(i=0;i<ispages_free;i++) {
	spage=(bk_ptr->spage_rp+i)%S_PAGES;
#ifdef PKDEB
	dev_info(&dev->dev, "PK pushing: spage: %d for link: %d\n",spage,link);
#endif
	iowrite32 (bk_ptr->spt[link][spage].spage_bus_addr_high, bk_ptr->common->bar0_regs+DFC_CNTLR_BASE_ADDR+16*link+4); 
	iowrite32 (bk_ptr->spt[link][spage].spage_bus_addr_low,	 bk_ptr->common->bar0_regs+DFC_CNTLR_BASE_ADDR+16*link+8);
	wmb();
	iowrite32 (bk_ptr->s_page_slot_number,	 			 bk_ptr->common->bar0_regs+DFC_CNTLR_BASE_ADDR+16*link+12);

      } //for i (spage)					
  } //for link		 	
  bk_ptr->spage_rp=((bk_ptr->spage_rp+ispages_free)%S_PAGES);
  atomic_set(&bk_ptr->spages_free,0);
  return(0);
                    }
		    

//---------------------------------------------------------------------------------------------------------------------------------
		    
static void altera_pcie40_set_drvdata(struct pci_dev *pdev, struct pcie40_dma_state *state)
{
  struct pcie40_state *common = pci_get_drvdata(pdev);
  common->dma_state = state;
  common->dma_state->common = common;
}
static struct pcie40_dma_state *altera_pcie40_get_drvdata(struct pci_dev *pdev)
{
  struct pcie40_state *common = pci_get_drvdata(pdev);
  return common->dma_state;
}
//---------------------------------------------------------------------------------------------------------------------------------

int  altera_pci_probe(struct pci_dev *dev, const struct pci_device_id *id)
{
    int rc = 0;
    struct pcie40_dma_state *bk_ptr = NULL;
    struct pcie40_state *common;
    void * __iomem base_bar0;

    bk_ptr = kzalloc(sizeof(struct pcie40_dma_state), GFP_KERNEL);
    if(!bk_ptr)
        goto err_bk_alloc;

    bk_ptr->pci_dev = dev;
    altera_pcie40_set_drvdata(dev, bk_ptr);
    common = pci_get_drvdata(dev);
    bk_ptr->common = common;
  
    base_bar0 =common->bar0_regs;

    rc = init_chrdev(bk_ptr); 
    if (rc) {
        dev_err(&dev->dev, "init_chrdev() failed\n");
        goto err_initchrdev;
    }
    rc = pci_enable_device(dev);
    if (rc) {
        dev_err(&dev->dev, "pci_enable_device() failed\n");
        goto err_enable;
    } else {
        dev_info(&dev->dev, "pci_enable_device() successful");
    }
    //rc = pci_request_regions(dev, P40_DRV_NAME);
    rc = pci_request_selected_regions_exclusive(dev, DMA_BARS, P40_DRV_NAME);
    if (rc) {
        dev_err(&dev->dev, "pci_request_regions() failed\n");
        goto err_regions;
          }
    printk(P40_DIAG "  pci request ok\n", P40_PARM);
    pci_set_master(dev);
    rc = pci_enable_msi(dev);
    if (rc) {
        dev_info(&dev->dev, "pci_enable_msi() failed\n");
        bk_ptr->msi_enabled = 0;
    } else {
        dev_info(&dev->dev, "pci_enable_msi() successful\n");
        bk_ptr->msi_enabled = 1;
    }
    pci_read_config_byte(dev, PCI_REVISION_ID, &bk_ptr->revision);
    pci_read_config_byte(dev, PCI_INTERRUPT_PIN, &bk_ptr->irq_pin);
    pci_read_config_byte(dev, PCI_INTERRUPT_LINE, &bk_ptr->irq_line);

    if (!pci_set_dma_mask(dev, DMA_BIT_MASK(64))) {
        pci_set_consistent_dma_mask(dev, DMA_BIT_MASK(64));
        dev_info(&dev->dev, "using a 64-bit irq mask\n");
    } else {
        dev_info(&dev->dev, "unable to use 64-bit irq mask\n");
        goto err_dma_mask;
    }

    dev_info(&dev->dev, "irq pin: %d\n", bk_ptr->irq_pin);
    dev_info(&dev->dev, "irq line: %d\n", bk_ptr->irq_line);
    dev_info(&dev->dev, "irq: %d\n", dev->irq);

   
    //  scan_bars(bk_ptr, dev);
    //map_bars(bk_ptr, dev);

    // waitqueue for user process
    init_waitqueue_head(&bk_ptr->wait_q); //LHCB

   init_waitqueue_head(&bk_ptr->wait_r);
  init_waitqueue_head(&bk_ptr->wait_w);
    
  sema_init(&bk_ptr->sem,1);
    // set default settings to run

// ------------------- SPAGE STUFF
   
  bk_ptr->s_page_slot_size 	= S_PAGE_SLOT_SIZE;
  bk_ptr->s_page_slot_number 	= S_PAGE_SLOT_NMB;
  bk_ptr->s_pages 			= S_PAGES;
  bk_ptr->link_number			= LINK_NUMBER;
  bk_ptr->link_mask			= LINK_MASK;
    
  bk_ptr->spage_rp=0;
  bk_ptr->spage_wp=0;
  atomic_set(&bk_ptr->spages_free,S_PAGES);
  atomic_set(&bk_ptr->spages_ready,0);
  
  //----
  {
    int i,j;
    for(i=0;i<LINK_NUMBER;i++)
      for(j=0;j<S_PAGES;j++) {			
	bk_ptr->spt[i][j].spage_virt_addr = pci_alloc_consistent(dev,S_PAGE_SIZE, &bk_ptr->spt[i][j].spage_bus_addr);
	memset(	bk_ptr->spt[i][j].spage_virt_addr,0x01,S_PAGE_SIZE);
	if (!bk_ptr->spt[i][j].spage_virt_addr) {
        rc = -ENOMEM;
	  goto err_spage_buffer;
    }
	bk_ptr->spt[i][j].spage_bus_addr_high=(bk_ptr->spt[i][j].spage_bus_addr)>>32;
	bk_ptr->spt[i][j].spage_bus_addr_low=bk_ptr->spt[i][j].spage_bus_addr;	
	dev_info(&dev->dev, "link: %d spage: %d virt: %llx  bus:%llx",i,j,(long long unsigned)bk_ptr->spt[i][j].spage_virt_addr,(long long unsigned)bk_ptr->spt[i][j].spage_bus_addr);	
    }
    }
    


    return 0;
    // error clean up
 err_spage_buffer:
  dev_err(&dev->dev, "goto err_spage_buffer");
  {
    int i,j;
    for(i=0;i<LINK_NUMBER;i++)
      for(j=0;j<S_PAGES;j++) 	
	pci_free_consistent(dev, S_PAGE_SIZE, bk_ptr->spt[i][j].spage_virt_addr, bk_ptr->spt[i][j].spage_bus_addr);
  }
   
err_dma_mask:
    dev_err(&dev->dev, "goto err_dma_mask");
    pci_release_regions(dev);
err_regions:
    
    pci_disable_device(dev);
err_enable:
    dev_err(&dev->dev, "goto err_enable");
    unregister_chrdev_region (bk_ptr->cdevno, 1);
err_initchrdev:
    dev_err(&dev->dev, "goto err_initchrdev");
    kfree(bk_ptr);
err_bk_alloc:
    dev_err(&dev->dev, "goto err_bk_alloc");
    return rc;
}
//-------------------------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------------------------

void  altera_pci_remove(struct pci_dev *dev)
{
    struct pcie40_dma_state *bk_ptr = NULL;
    bk_ptr = altera_pcie40_get_drvdata(dev);
    
    device_destroy(pcie40_dma_class, MKDEV(MAJOR(bk_ptr->cdevno), MINOR(bk_ptr->cdevno)));
    cdev_del(&bk_ptr->cdev);
    unregister_chrdev_region(bk_ptr->cdevno, 1);
    pci_disable_device(dev);
    if(bk_ptr) {
        if(bk_ptr->msi_enabled) {
            pci_disable_msi(dev);
            bk_ptr->msi_enabled = 0;
        }
    }
    //unmap_bars(bk_ptr, dev);
    pci_release_regions(dev);


    kfree(bk_ptr);
}

int  altera_dma_init(void)
{
    int rc = 0;

   
  pcie40_dma_class = class_create(THIS_MODULE, PCIE40_DMA_CLASS);
  if (IS_ERR(pcie40_dma_class)) {
    rc = PTR_ERR(pcie40_dma_class);
    printk(P40_WARN "failed to register class, %d\n", P40_PARM, rc);
    goto err_class_create;
  }
  //pcie40_ecs_class->dev_uevent = pcie40_dev_uevent;
  pcie40_dma_class->devnode = pcie40_devnode;
  return rc;

err_class_create:
  return rc;

   
}

void altera_dma_exit(void)
{
  class_destroy(pcie40_dma_class);
  
}

