#include "pcie40_reg.h"
#include <unistd.h>
#include <stdio.h>
#include <vector>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include <iostream>
// ===== 
// CALL C functions
// =====

#ifndef D
#define D(a,b,c) (((a)>>(c))&((1<<((b)+1-(c)))-1))
#define B(a,b)   D(a,b,b)
#define Bs(a,b,s)   (B(a,b)?(s):"")
#define Ds(a,b,c,s)   (D(a,b,c)?(s):"")
#endif

extern "C" int      ecs_write(int dev, int bar, unsigned add, int val); 
extern "C" unsigned ecs_read(int dev, int bar, unsigned add) ; 
extern "C" int      ecs_openLli(int dev);
extern "C" int      ecs_iordLli(int dev, unsigned add, unsigned *val);
extern "C" void     ecs_closeLli(int dev); 

#define SIZE_MEMO_FPGA 1024 // maximum size of memory in FPGA

int pcie40_status(int dev){
  int ret = -1;
  ret = ecs_read(dev, SLC_BAR,  SLC_PCIE40_STATUS);
  return ret;
}


const char* pcie40_fwver(int dev){

  int board = 0;
  unsigned val;
  int reg = 0;
  char contenuMemoireFPGA[SIZE_MEMO_FPGA + 2];
  
  for(int i=0; i < SIZE_MEMO_FPGA; i++){
    if (ecs_iordLli(board, reg, &val)!=0){
      exit(0);
    }
    contenuMemoireFPGA[i] = val;
    reg += 4;    
  }
  contenuMemoireFPGA[SIZE_MEMO_FPGA+1] = '\0';
  //printf("contenu memoire FPGA   %s\n",contenuMemoireFPGA);

  std::string memo_str = contenuMemoireFPGA;
  char delims = '"'; 
  unsigned int first = 0;
  unsigned int last = memo_str.find_first_of(delims);
  
  std::vector<std::string> memo;
  
  while (first < memo_str.size()) {
    std::string subStr(memo_str, first, last - first);
 
    memo.push_back(subStr);
    
    first = last + 1;
    last = memo_str.find_first_of(delims, first);
    
    if (last == std::string::npos) {
      last = memo_str.size();
    }
  }

  //for(int i = 0; i < memo.size(); i++){
  //std::cout <<i<<" " <<memo[i] << std::endl;
  //}

  return memo[23].c_str();
}

int pcie40_run_number(int dev){
  int ret = -1;
  ret = ecs_read(dev, SLC_BAR,  SLC_RUM_NUMBER);
  return (ret & 0xFF00) >> 8 | (ret & 0x00FF) << 8;
}

int pcie40_trgtag_number(int dev){
  int ret = -1;
  ret = ecs_read(dev, SLC_BAR,  SLC_TRGTAG_NUMBER);
  return ret;
}

int pcie40_memo_status(int dev){
  return B(pcie40_status(dev), 8);
}

int pcie40_ttd_status(int dev){
  return B(pcie40_status(dev), 12);
}

int pcie40_ttdclk_status(int dev){
  return B(pcie40_status(dev), 11);
}

int pcie40_trg_number(int dev){
  return D(pcie40_status(dev), 28, 17);
}

int pcie40_trg_type(int dev){
  return D(pcie40_status(dev), 16, 13);
}

int pcie40_rxlink_status(int dev){
  return B(pcie40_status(dev), 0);
}

int pcie40_txlink_status(int dev){
  return B(pcie40_status(dev), 1);
}

int pcie40_fee_synced(int dev){
  return B(pcie40_status(dev), 2);
}

int pcie40_disp_status(int dev){
  return B(pcie40_status(dev), 3);
}

int pcie40_rxdata_error(int dev){
  return B(pcie40_status(dev), 4);
}

//----------------------------------------------
// Belle2link dedicated
//----------------------------------------------
int pcie40_b2l_status(int dev, int ch){
  int ret = -1;
  if(ch >= 0 && ch <=23)
    ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_STATUS1);
  else if (ch >= 24 && ch <= 47 )
    ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_STATUS2);
  
  return ( ( ret & ( 1 << ( SLC_BASE_BIT + ch ) ) ) 
           >> ( SLC_BASE_BIT + ch ));
}

int pcie40_b2l_mask(int dev, int ch){
  int ret = -1;
  if(ch >= 0 && ch <=23)
    ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_MASK1);
  else if (ch >= 24 && ch <= 47 )
    ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_MASK2);
  
  return ( ( ret & ( 1 << ( SLC_BASE_BIT + ch ) ) ) 
           >> ( SLC_BASE_BIT + ch ));
}

int pcie40_b2l_txstatus(int dev, int ch){
  int ret = -1;
  if(ch >= 0 && ch <=23)
    ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_TX_STATUS1);
  else if (ch >= 24 && ch <= 47 )
    ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_TX_STATUS2);
  
  return ( ( ret & ( 1 << ( SLC_BASE_BIT + ch ) ) ) 
           >> ( SLC_BASE_BIT + ch ));
}

int pcie40_b2l_rxstatus(int dev, int ch){
  int ret = -1;
  if(ch >= 0 && ch <=23)
    ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_RX_STATUS1);
  else if (ch >= 24 && ch <= 47 )
    ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_RX_STATUS2);
  
  return ( ( ret & ( 1 << ( SLC_BASE_BIT + ch ) ) ) 
           >> ( SLC_BASE_BIT + ch ));
}

int pcie40_b2l_txready(int dev, int ch){
  int ret = -1;
  if(ch >= 0 && ch <=23)
    ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_TX_READY1);
  else if (ch >= 24 && ch <= 47 )
    ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_TX_READY2);
  
  return ( ( ret & ( 1 << ( SLC_BASE_BIT + ch ) ) ) 
           >> ( SLC_BASE_BIT + ch ));
}

int pcie40_b2l_rxready(int dev, int ch){
  int ret = -1;
  if(ch >= 0 && ch <=23)
    ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_RX_READY1);
  else if (ch >= 24 && ch <= 47 )
    ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_RX_READY2);
  
  return ( ( ret & ( 1 << ( SLC_BASE_BIT + ch ) ) ) 
           >> ( SLC_BASE_BIT + ch ));
}

int pcie40_b2l_cnt_vetoidle(int dev, int ch){
  int ret = -1;
  ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_VETOIDLE + ch*0x20);
  
  return ( ret & 0xFFFF);
}

int pcie40_b2l_cnt_vetodata(int dev, int ch){
  int ret = -1;
  ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_VETODATA + ch*0x20);
  
  return ( ret & 0xFFFF);
}

int pcie40_b2l_3rdrxdata(int dev, int ch){
  int ret = -1;
  ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_BUFDOWN0 + ch*0x20);
  return (ret & 0xFF);
}

int pcie40_b2l_2ndrxdata(int dev, int ch){
  int ret = -1;
  ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_BUFDOWN0 + ch*0x20);
  return ((ret & 0xFF00) >> 8);
}

int pcie40_b2l_1strxdata(int dev, int ch){
  int ret = -1;
  ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_BUFDOWN1 + ch*0x20);
  return (ret & 0xFF);
}

int pcie40_b2l_3rdcharisk(int dev, int ch){
  int ret = -1;
  ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_BUFDOWN1 + ch*0x20);
  return ((ret & 0x300) >> 8);
}

int pcie40_b2l_2ndcharisk(int dev, int ch){
  int ret = -1;
  ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_BUFDOWN1 + ch*0x20);
  return ((ret & 0xC00) >> 10);
}

int pcie40_b2l_1stcharisk(int dev, int ch){
  int ret = -1;
  ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_BUFDOWN1 + ch*0x20);
  return ((ret & 0x3000) >> 12);
}

int pcie40_b2l_cnt_linkdown(int dev, int ch){
  int ret = -1;
  ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_CNTDOWN + ch*0x20);
  
  return ( ret & 0xFFFF);
}

int pcie40_b2l_monstatus(int dev, int ch){
  int ret = -1;
  return ret = ecs_read(dev, SLC_BAR,  SLC_B2LINK_MONSTATUS + ch*0x20); 
}

int pcie40_b2l_vaild_channel(int dev, int ch){
  return B(pcie40_b2l_monstatus(dev, ch), 1);
}

int pcie40_b2l_usebadidle(int dev, int ch){
  return B(pcie40_b2l_monstatus(dev, ch), 2);
}

int pcie40_b2l_usebaddata(int dev, int ch){
  return B(pcie40_b2l_monstatus(dev, ch), 3);
}

int pcie40_b2l_sta_linkdown(int dev, int ch){
  return B(pcie40_b2l_monstatus(dev, ch), 4);

}








