/*
  hsreg.h

  PCIE40 registers


*/

#ifndef __PICE40REG_H__
#define __PICE40REG_H__

#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <iostream>

#if defined(__cplusplus)
extern "C" {
#endif
#if defined(__dummy_close_bracket_to_cheat_emacs_auto_indent)
}
#endif

#define SLC_BAR 2

#define SLC_BASE_BIT 0
#define SLC_B2LINK_RX_READY1   0x000500E0  // link in the ready state for rxdata, channel 0-23, 1 bit pre channel
#define SLC_B2LINK_RX_READY2   0x00050100  // link in the ready state for rxdata, channel 0-23, 1 bit pre channel
#define SLC_B2LINK_TX_READY1   0x00050120  // link in the ready state for txdata, channel 0-23, 1 bit pre channel
#define SLC_B2LINK_TX_READY2   0x00050140  // link in the ready state for txdata, channel 0-23, 1 bit pre channel
#define SLC_B2LINK_RX_STATUS1  0x00050160  // link in the ready state for txdata, channel 0-23, 1 bit pre channel
#define SLC_B2LINK_RX_STATUS2  0x00050180  // link in the ready state for txdata, channel 0-23, 1 bit pre channel
#define SLC_B2LINK_TX_STATUS1  0x000501A0  // link in the ready state for txdata, channel 0-23, 1 bit pre channel
#define SLC_B2LINK_TX_STATUS2  0x000501C0  // link in the ready state for txdata, channel 0-23, 1 bit pre channel
#define SLC_B2LINK_VETOIDLE    0x00070010  // count vetoidle
#define SLC_B2LINK_VETODATA    0x00070000  // count vetodata
#define SLC_B2LINK_BUFDOWN0    0x00090000  // buffer recent two words of rxdata
#define SLC_B2LINK_BUFDOWN1    0x00080000  // buffer recent two words of rxdata
#define SLC_B2LINK_CNTDOWN     0x000A0010  // buffer recent third words of rxdata [15 downto 0] + three charish {21 downto 16} 
#define SLC_B2LINK_MONSTATUS   0x000A0000  // buffer recent two words of rxdata

#define SLC_B2LINK_STATUS1     0x000501E0  // link up/down channel 0-23, 1 bit pre channel
#define SLC_B2LINK_STATUS2     0x00050200  // link up/down channel 24-47, 1 bit pre channel
#define SLC_B2LINK_MASK1       0x00050520  // link up/down channel 0-23, 1 bit pre channel
#define SLC_B2LINK_MASK2       0x00050540  // link up/down channel 24-47, 1 bit pre channel
#define SLC_RFIFO_STATUS       0x00060010


#define SLC_PCIE40_STATUS      0x00050000 
#define SLC_RUM_NUMBER         0x00050020
#define SLC_TRGTAG_NUMBER      0x00050040


int pcie40_b2l_status(int dev, int ch);
int pcie40_b2l_txstatus(int dev, int ch);
int pcie40_b2l_rxstatus(int dev, int ch);
int pcie40_b2l_txready(int dev, int ch);
int pcie40_b2l_rxready(int dev, int ch);
int pcie40_b2l_mask(int dev, int ch);
int pcie40_b2l_cnt_vetoidle(int dev, int ch);
int pcie40_b2l_cnt_vetodata(int dev, int ch);
int pcie40_b2l_3rdrxdata(int dev, int ch);
int pcie40_b2l_2ndrxdata(int dev, int ch);
int pcie40_b2l_1strxdata(int dev, int ch);
int pcie40_b2l_3rdcharisk(int dev, int ch);
int pcie40_b2l_2ndcharisk(int dev, int ch);
int pcie40_b2l_1stcharisk(int dev, int ch);
int pcie40_b2l_cnt_linkdown(int dev, int ch);
int pcie40_b2l_monstatus(int dev, int ch);
int pcie40_b2l_vaild_channel(int dev, int ch);
int pcie40_b2l_usebadidle(int dev, int ch);
int pcie40_b2l_usebaddata(int dev, int ch);
int pcie40_b2l_sta_linkdown(int dev, int ch);
int pcie40_status(int dev);
const char* pcie40_fwver(int dev);
int pcie40_run_number(int dev);
int pcie40_trgtag_number(int dev);
int pcie40_trg_type(int dev);
int pcie40_memo_status(int dev);
int pcie40_ttd_status(int dev);
int pcie40_ttdclk_status(int dev);
int pcie40_trg_number(int dev);
int pcie40_rxlink_status(int dev);
int pcie40_txlink_status(int dev);
int pcie40_fee_synced(int dev);
int pcie40_disp_status(int dev);
int pcie40_rxdata_error(int dev);



/* FEE registers as implemented in belle2link_cdc */

#define PCIE40REG_SERIAL    0x00
#define PCIE40REG_PDSTL     0x01 /* 0x01 .. 0x60 for 48 ch (x2) */
#define PCIE40REG_PDSTLMIN  0x01
#define PCIE40REG_PDSTLMAX  0x60
#define PCIE40REG_ADCTH     0x61 /* same as lower 8 bit of ADC threshold setting */
#define PCIE40REG_ADCL      0x61
#define PCIE40REG_ADCH      0x62
#define PCIE40REG_WINDOW    0x63
#define PCIE40REG_DELAY     0x64
#define PCIE40REG_TDCTH     0x65 /* same as lower 8 bit of DAC setting for TDC */
#define PCIE40REG_DACL      0x65
#define PCIE40REG_DACH      0x66
#define PCIE40REG_CDCCONT   0x67 /* part of b2ldo (FEE_CONTROL) moved from 0x69 */

/* FEE registers for Belle2link internal use */

#define PCIE40REG_CRCERR    0x68
#define PCIE40REG_FEECONT   0x69
#define PCIE40REG_D32A      0x6a
#define PCIE40REG_D32B      0x6b
#define PCIE40REG_D32C      0x6c
#define PCIE40REG_D32D      0x6d

/* HSLB registers in addition to predefined FINESSE registers */

#define PCIE40REG_D32       0x6e
#define PCIE40REG_A32       0x6f
#define PCIE40REG_STAT      0x72   /* b2ldo (STATUS) */

#define PCIE40REG_CCLK      0x74   /* cpld */
#define PCIE40REG_CONF      0x75   /* cpld */
#define PCIE40REG_CPLDVER   0x76   /* cpld */
#define PCIE40REG_FEEHWTYPE 0x77
#define PCIE40REG_FEESERIAL 0x78
#define PCIE40REG_FEEFWTYPE 0x79
#define PCIE40REG_FEEFWVER  0x7a

#define B2LFEE_UNDEF   0
#define B2LFEE_SVD     1
#define B2LFEE_CDC     2
#define B2LFEE_BPID    3
#define B2LFEE_EPID    4
#define B2LFEE_ECL     5
#define B2LFEE_KLM     6
#define B2LFEE_TRG     7
#define B2LFEE_TEST    15


/* HSLB 32-bit register extension */

#define PCIE40REGL_ID      0x80
#define PCIE40REGL_VER     0x81
#define PCIE40REGL_RESET   0x82
#define PCIE40REGL_STAT    0x83
/*
  -- input
  stal(16#81#)    <= x"0000" & std_logic_vector(to_unsigned(VERSION, 16));

  stal(16#83#)(0) <= not linkup;
  stal(16#83#)(1) <= disable;
  stal(16#83#)(2) <= led3;  -- FFUL
  stal(16#83#)(3) <= led4;  -- LinkError
  stal(16#83#)(4) <= led5;  -- NWFF
  stal(16#83#)(5) <= sta_bad127;
  stal(16#83#)(6) <= not plllk;
  stal(16#83#)(7) <= not stapll2;
  stal(16#83#)(11 downto  8) <= stateff;
  stal(16#83#)(15 downto 12) <= staterx;
  stal(16#83#)(19 downto 16) <= statepr;
  stal(16#83#)(23 downto 20) <= statept;
  stal(16#83#)(28 downto 24) <= statetx;
  stal(16#83#)(29) <= not stapll;
  stal(16#83#)(30) <= not fwenb;
  stal(16#83#)(31) <= stalinkdown;

  stal(16#84#) <= cntlinkdown(7 downto 0) & cntcrcerr(7 downto 0) & rxdata;
  stal(16#85#) <= cnt_evtsz & cnt_maxsz;
  stal(16#86#) <= cnt_event;  -- number of events into copper fifo
  stal(16#87#) <= cnt_sec127;

  stal(16#88#) <= cnt_nword(39 downto 8);

  stal(16#89#)(31 downto 28) <= cnt_oflow;
  stal(16#89#)(27 downto 24) <= cnt_badalign;
  stal(16#89#)(23 downto 20) <= cnt_realign;
  stal(16#89#)(19) <= badalign;
  stal(16#89#)(18) <= rxrealign;
  stal(16#89#)(17) <= not rxaligned;
  stal(16#89#)(16) <= not led6;  -- sta_rxbyteisaligned
  stal(16#89#)(15 downto  8) <= cnt_fwclk;
  stal(16#89#)(7  downto  0) <= cnt_nword(7 downto 0);

  stal(16#8c#)(31 downto 16) <= cntvetoidle;
  stal(16#8c#)(15 downto 0)  <= cntvetodata;
  stal(16#8d#)(25 downto 24) <= bufvetodown(53 downto 52);
  stal(16#8d#)(21 downto 20) <= bufvetodown(51 downto 50);
  stal(16#8d#)(17 downto 16) <= bufvetodown(49 downto 48);
  stal(16#8d#)(15 downto 0)  <= bufvetodown(47 downto 32);
  stal(16#8e#)(31 downto 0)  <= bufvetodown(31 downto  0);

  stal(16#8f#) <= dbg;

  -- output
  setrunreset  <= regl(16#82#)(0);
  b2lreset     <= regl(16#82#)(4);
  txreset      <= regl(16#82#)(8);
  gtpreset     <= regl(16#82#)(12);
  clrpll       <= regl(16#82#)(16);
  clrpll2      <= regl(16#82#)(20);
  usebadidle   <= regl(16#8b#)(1);
  usebaddata   <= regl(16#8b#)(0);
*/
#define PCIE40REGL_RXDATA  0x84  /* stal(16#84#) <= cntcrcerr & rxdata; */
#define PCIE40REGL_EVENTSZ 0x85
#define PCIE40REGL_NEVENT  0x86
#define PCIE40REGL_UPTIME  0x87
#define PCIE40REGL_NKBYTE  0x88
#define PCIE40REGL_NWORD   0x89
#define PCIE40REGL_VETOSET 0x8b
#define PCIE40REGL_VETOCNT 0x8c
#define PCIE40REGL_VETOBUF0 0x8d
#define PCIE40REGL_VETOBUF1 0x8e
#define PCIE40REGL_CNTFF00     0x95 /* new in hslb055 */
#define PCIE40REGL_CNTDATAPKT  0x96 /* new in hslb055 */
#define PCIE40REGL_CNTDATAWORD 0x97 /* new in hslb055 */

static const int hsreg_revision = 1;

struct pcie40reg {
  uint32_t  stat;    /* 72 */
  uint8_t  csr;     /* 73 */
  uint8_t  xbusy;   /* 74 */
  uint8_t  conf;    /* 75 */
  uint8_t  feehw;   /* 77 >> 4 */
  uint8_t  feefw;   /* 79 >> 4 */
  uint8_t  feecrce; /* 68 */
  uint16_t feeser;  /* 78 | ((77 & f) << 8) */
  uint16_t feever;  /* 7a | ((79 & f) << 8) */
  uint32_t pcie40id;  /* 80 */
  uint32_t pcie40ver; /* 81 */
  uint32_t pcie40rst; /* 82 */
  uint32_t pcie40sta; /* 83 */
  uint32_t rxdata;  /* 84 */
  uint32_t eventsz; /* 85 */
  uint32_t nevent;  /* 86 */
  uint32_t uptime;  /* 87 */
  uint32_t nkbyte;  /* 88 */
  uint32_t nword;   /* 89 */
  uint32_t vetoset; /* 8b */
  uint32_t vetocnt; /* 8c */
  uint32_t vetobuf[2]; /* 8d, 8e */
  uint32_t cntff00;     /* 95 */
  uint32_t cntdatapkt;  /* 96 */
  uint32_t cntdataword; /* 97 */
};

typedef struct pcie40reg pcie40reg_t;


int pcie40reg_read(int fd /* 0-3 */, pcie40reg_t*);
int pcie40reg_getfee(int fd /* 0-3 */, pcie40reg_t*);

/* ---------------------------------------------------------------------- *\
 *    feename
 *    
\* ---------------------------------------------------------------------- */
const char *
feename(int hwtype, int fwtype)
{
  static const char *feetype[] = {
    "UNDEF", "SVD", "CDC", "TOP", "ARI", "ECL", "EECL", "KLM", "EKLM",
    "TRG", "FEE10", "FEE11", "FEE12", "FEE13", "DEMO", "TEST" };
  static const char *demotype[] = {
    "UNDEF", "HSLB-B2L", "SP605-B2L", "ML605-B2L", "AC701-B2L" };
  static const char *trgtype[] = {
    "TRGMERGER", /* = 0 */
    "TRGTSF",    /* = 1 */
    "TRG2D",     /* = 2 */
    "TRG3D",     /* = 3 */
    "TRGNN",     /* = 4 */
    "TRGEVTT",   /* = 5 */
    "TRGGRL",    /* = 6 */
    "TRGGDL",    /* = 7 */
    "TRGETM",    /* = 8 */
    "TRGTOP",    /* = 9 */
    "TRGKLM",    /* = 10 */
  };

  hwtype &= 0x0f;
  fwtype  %= 16;

  if (hwtype == 14 && fwtype > 0 && fwtype <= 4) {
    return demotype[fwtype];
  } else if (hwtype == 9 && fwtype >= 0 && fwtype <= 10) {
    return trgtype[fwtype];
  } else {
    int i;
    static int ringi = 0;
    static int ringval[32];
    static char ringbuf[32][20];
    for (i=0; i<32; i++) {
      if (ringval[i] == 0x100 + (hwtype<<4) + fwtype) break;
    }
    if (i == 32) {
      i = ringi;
      ringval[i] = 0x100 + (hwtype<<4) + fwtype;
      sprintf(ringbuf[i], "%s", feetype[hwtype]);
      ringi = (ringi + 1) % 32;
    }
    return ringbuf[i];
  }
}

#if defined(__dummy_open_bracket_to_cheat_emacs_auto_indent)
__dummy_open_bracket_to_cheat_emacs_auto_indent {
#endif
#if defined(__cplusplus)
}
#endif

#endif /* __PCIE40REG_H__ */

